/**************************************************************************************************/

/* Function Prototypes */
static char *SaveTarga(char *name, struct IconImage *im, int bpp);
static char *WriteTruecolor(BPTR file, struct IconImage *im, int bpp, BOOL tgacomp);

/**************************************************************************************************/

static char *SaveTarga(char *name, struct IconImage *im, int bpp)
{
    BPTR filehandle;
    int width, height, depth;	
    char *error = NULL;
    char *status = NULL;
	BOOL tgacomp = FALSE;        
	ULONG numcolors = 0;
	ULONG alignwidth = 0;
    	
    if (filehandle = Open (name, MODE_NEWFILE))
    {         
        width = im->Width;
        height = im->Height;
        depth = im->Depth;
    
	    /* disallow images with depth not eq to 8, 16, 24 or 32 */
        //if (depth != 8 && depth != 16 && depth != 24 && depth != 32) 
        if (depth != 24 && depth != 32)    
	    {
		    printf("SaveTarga() --- color depth %d, can save only depths of 24 or 32", depth);		    
		    error = "ERROR_NOT_IMPLEMENTED";		    
	    }
	    printf("SaveTarga() --- Picture size %d x %d (x %d bit)\n", width, height, depth);
        
	    printf("SaveTarga() --- Setup Pixel Data by BitDepth.\n");

	    /* Set Default Data for 24bit */
	    int cmaptype = 0;
	    int imagetype = 2;
	    int cmapdepth = 0;
	    int compressed = 0;
	    //int compressed = 1;        
        
        /* Set compression value */
	    if(compressed == 1) 
	    {
		    imagetype += 8;
		    tgacomp = TRUE;
	    }

        /* Set Values for new TargaHeader */
        UBYTE TGAHDR[18];
        TGAHDR[0] = 0;
        TGAHDR[1] = cmaptype;
        TGAHDR[2] = imagetype;
        TGAHDR[3] = TGAHDR[4] = 0;    
	    TGAHDR[5] = (UBYTE)(numcolors & 0xFF);
        TGAHDR[6] = (UBYTE)(numcolors >> 8);  
        TGAHDR[7] = cmapdepth; 
        TGAHDR[8] = TGAHDR[9] = 0;    
        TGAHDR[10] = TGAHDR[11] = 0; 
        TGAHDR[12] = (UBYTE)(width & 0xFF);
        TGAHDR[13] = (UBYTE)(width >> 8);    
        TGAHDR[14] = (UBYTE)(height & 0xFF);
        TGAHDR[15] = (UBYTE)(height >> 8); 
        TGAHDR[16] = depth; 
        TGAHDR[17] = 0;     
         
	    /* Write new targa header to File */
        if( FWrite( filehandle, TGAHDR, 18, 1 ) != 1 )
        {
            printf("SaveTarga() --- writing targa header failed !\n");
		    error = "NOT OK";
        }
	    printf("SaveTarga() --- writing targa header successful.\n");
                
        printf("SaveTarga() --- begin writing picture data\n");	    
        
	    if((depth == 24) || (depth == 32))
        { 
		    printf("SaveTarga() --- writing to file (24/32bit mode).\n");            			            			
		    status = WriteTruecolor(filehandle, im, bpp, tgacomp);            
		    if(status != "OK")		
		    {			    
			    error = "ERROR_OBJECT_WRONG_TYPE";			    
		    }
        }        
        printf("SaveTarga() --- Normal Exit\n");
    }
    if (filehandle) Close (filehandle);
	
    return error;       
}

/**************************************************************************************************/

static char *WriteTruecolor(BPTR file, struct IconImage *im, int bpp, BOOL tgacomp)
//char *WriteTruecolor(Class *cl, Object *o, BPTR file, UBYTE *linebuf, struct BitMapHeader *bmh, BOOL tgacomp)
{
    /*
    ** Write 24 or 32-bit Scanline
    ** stored in a 24/32bit image file
    */
    
	int y; 
    int idx;	
	int width;
	int height;
	int depth;
	int yoffset;    
    LONG length;
	UBYTE *buf;	
	UBYTE a,r,g,b;
    UBYTE *linebuf;	
    char *error = NULL;
    
    width = im->Width;
    height = im->Height;
    depth = im->Depth;
    length = width*bpp; 
	yoffset = height - 1;    		
	
    /* Allocate memory for scanline data */ 
	linebuf = AllocMem(length, MEMF_ANY);
	if(!linebuf)
	{        
		error = "ERROR_NO_FREE_STORE";
		goto exit;		    
    }

    buf = AllocMem(length, MEMF_ANY);
	if (!buf) 
	{
		error = "ERROR_NO_FREE_STORE";		
        goto exit;
	}

    printf("\nWriteTrueColor(): Beginning Writing. (24/32bit mode)\n");

    /* Write ScanLine Data from Bottom to Top for TGA */       
	for (y=0; y<height; y++) //Use yoffset to write to the bottom of the file.	
    {		
		/* Get single line of image */            
        if(!memcpy(buf, im->ImageData+(length*yoffset), length)) 
        {
            printf("WriteTrueColor() ---  copy scanline %d failed !\n", y);                
		    error = "READPIXELARRAY FAILED";
            goto exit;
        }       

		/* Process scanline data for 24bit or 32bit */
		if (bpp == 3) 
		{						
			/* Reverse 'RGB' to 'BGR' for TGA file for 24bit */				
            for (idx = 0; idx < (width * bpp); idx+=3)
			{
				r = buf[idx];
				g = buf[idx + 1];
				b = buf[idx + 2];
				linebuf[idx] = b;
				linebuf[idx + 1] = g;
				linebuf[idx + 2] = r;                
			}
		} 
		else if (bpp == 4)
		{			
			/* Reverse 'ARGB' to 'BGRA' for TGA file for 32bit */
            for (idx = 0; idx < (width * bpp); idx+=4)
			{	 
				a = buf[idx];
				r = buf[idx + 1];
				g = buf[idx + 2];
				b = buf[idx + 3];
				linebuf[idx] = b;
				linebuf[idx + 1] = g;
				linebuf[idx + 2] = r;
				linebuf[idx + 3] = a;				
			}
		}
        
        if(tgacomp == TRUE)
		{
			if (bpp == 3)
		    {
			    /* Reverse 'RGB' to 'BGR' for TGA file for 24bit */				
                for (idx = 0; idx < (width * bpp); idx+=3)
			    {
				    r = buf[idx];
				    g = buf[idx + 1];
				    b = buf[idx + 2];
				    buf[idx] = b;
				    buf[idx + 1] = g;
				    buf[idx + 2] = r;                
			    }
			}
			else if (bpp == 4)
		    {			
			    /* Reverse 'ARGB' to 'BGRA' for TGA file for 32bit */
                for (idx = 0; idx < (width * bpp); idx+=4)
			    {	 
				    a = buf[idx];
				    r = buf[idx + 1];
				    g = buf[idx + 2];
				    b = buf[idx + 3];
				    linebuf[idx] = b;
				    linebuf[idx + 1] = g;
				    linebuf[idx + 2] = r;
				    linebuf[idx + 3] = a;				
			    }
		    }
			/* Write rle encoded scanline data to the output file */
			printf("targa.datatype/SaveTarga() --- writing rle data line: %d\n", y);
			//WriteRLE(file, buf, bmh->bmh_Width, bpp); //1,2,3,4
		}
		else
		{
            /* Write unencoded scanline data to the output file */
            if( FWrite( file, linebuf, length, 1 ) != 1 )
            {
                printf("targa.datatype/SaveTarga() --- writing picture scanline %d failed !\n", y);            
			    error = "WRITING PICTURE SCANLINE FAILED";
			    goto exit;
            }
        }
        yoffset--;
    }	

	exit:
	/* Free Resources */	
	if(buf) FreeMem(buf, length);    
	if(linebuf) FreeMem(linebuf, length);

	return error;
}

/**************************************************************************************************/